@@ -12,6 +12,13 @@
 
 #include <signal.h>
 #include <net/if.h>
+#ifndef IFNAMSIZ
+#  ifdef IF_NAMESIZE
+#    define IFNAMSIZ IF_NAMESIZE
+#  else
+#    define IFNAMSIZ 16
+#  endif
+#endif
 #ifdef HAVE_OPENSSL_SSL_H
 #include <openssl/ssl.h>
 #endif
@@ -330,6 +337,8 @@ void server_main(struct nsd *nsd);
 void server_child(struct nsd *nsd);
 void server_shutdown(struct nsd *nsd) ATTR_NORETURN;
 void server_close_all_sockets(struct nsd_socket sockets[], size_t n);
+const char* nsd_event_vs(void);
+const char* nsd_event_method(void);
 struct event_base* nsd_child_event_base(void);
 void service_remaining_tcp(struct nsd* nsd);
 /* extra domain numbers for temporary domains */
